#!/usr/bin/perl
# 
# This script should always be run from the SOURCEROOT directory,
# where SOURCEROOT is an environment variable, specifying the root
# of all source code directories.
#

$MAKE = "make";

sub dir_name 
{
	"$ENV{SOURCEROOT}/src/$_[0]/$_[1]";
}

sub foreach_module
{
    $headline = $_[0];
    $func     = $_[1];

    foreach $sec (@sections)
    {
        print "\n----------------------------------------\n";
        print "Entering section $sec\n";
        print "\n----------------------------------------\n\n";
        foreach $mod (@{$modules{$sec}})
        {
            print "--- $headline\'ing module $mod ---- \n";
            &$func($sec,$mod);
        }
        
    }
}

sub purge
{
    print "Purging ...\n";
    $dir = "$ENV{SOURCEROOT}/src/";
    print "Finding SGI specific, prelinker related garbage\n";
    @spurious_files = `find $dir -name 'ii_files'`;
    print "Finding temp file garbage\n";
    @spurious_files = (@spurious_files,`find $dir -name '*~'`);
    print "Finding dependency files\n";
    @spurious_files = (@spurious_files,`find $dir -name '*.d'`);
    print "Removing found files\n";
    foreach $sf (@spurious_files)
    {
        system "rm -fr $sf\n";
    }
}

sub get_system
  {
    $sys = `uname`;
  }

sub get_include_directories
{ 
    foreach (@sections)
    {
        $include_str = $include_str . " -I$ENV{SOURCEROOT}/src/$_";
    }
    $include_str;
}

$list_module = sub { print `ls &dir_name($_[0],$_[1])`; };

$clean_module = sub 
{ 
    $dir = &dir_name($_[0],$_[1]);
    print "No Makefile in $dir", next if ! -e "$dir/Makefile";
    system "$MAKE -k -C $dir clean";
};

$make_module = sub 
{
    $dir = &dir_name($_[0],$_[1]);
    print "No Makefile in $dir", next if ! -e "$dir/Makefile";
    system "$MAKE -k -C $dir";
};

$cvs_module = sub 
{
    $dir = &dir_name($_[0],$_[1]);
		system "cd $dir ; cvs add *.cc *.h";
};

$make_makefile_link = sub 
{
    $dir = &dir_name($_[0],$_[1]);
    if(! -e "$dir/Makefile")
    {
        system "ln -s $ENV{SOURCEROOT}/Makefile.lib $dir/Makefile";
    }
};

#----------------------------------------
# main
#----------------------------------------

@sections_raw = `cd $ENV{SOURCEROOT}/src; ls -1dp *`;
foreach (@sections_raw)
{
    next if $_ =~ /^CVS\//;
    @sections = (@sections,$1) if($_ =~ /(\w*)\/$/);
}
foreach $sec (@sections)
{
    @modules_raw = `cd $ENV{SOURCEROOT}/src/$sec; ls -1dp *`;
    undef @mods;
    foreach (@modules_raw)
    {
        next if $_ =~ /^CVS\//;
        @mods = (@mods,$1) if($_ =~ /(\S*)\/$/);
    }
    @{$modules{$sec}} = @mods;
}
#
# Default behaviour is to compile
#
if($#ARGV<0)
{
    foreach_module("Make", $make_module);
    exit;
}
#
# else ... for each argument
#
for($i=0;$i<=$#ARGV;$i++)
{
    print &get_include_directories if $ARGV[$i] =~ /^-includes/;

    print &get_system if $ARGV[$i] =~ /^-system/;

    foreach_module("List", $list_module), next if $ARGV[$i] =~ /^\-list/;

    foreach_module("Make", $make_module), next if $ARGV[$i] =~ /^\-remake/;
    
    if ($ARGV[$i] =~ /^\-pristine/)
    {
        foreach_module("Clean", $clean_module);
        &purge;
        next;
    }

    if ($ARGV[$i] =~ /^\-cvs/)
    {
        foreach_module("CVS'ing", $cvs_module);
        system "cvs commit";
        next;
    }
    
    foreach_module("Makefile link", $make_makefile_link), next
        if $ARGV[$i] =~ /^\-makefiles/;
}






