#ifndef __ARITHVEC_H
#define __ARITHVEC_H

#include <assert.h>
#include <iostream>

#include "VecDat.h"

#include "ArithVecApplicativeTemplates.h"

template <class T, class V, int N> 
class ArithVec: public VecDat<T,N>
{

public:

	typedef T ScalarType;
	typedef V VectorType;

	static int get_dim() {return N;}

  ArithVec(): VecDat<T,N>(T(0)) {}

  explicit ArithVec(T _a) :VecDat<T,N>(_a) {}
  ArithVec(T _a, T _b):VecDat<T,N>(_a,_b) {}
  ArithVec(T _a, T _b, T _c):VecDat<T,N>(_a,_b,_c) {}
  ArithVec(T _a, T _b, T _c, T _d):VecDat<T,N>(_a,_b,_c,_d) {}

  //----------------------------------------------------------------------
  // Misc.
  //----------------------------------------------------------------------

  static int dimension() {return N;}

  //----------------------------------------------------------------------
  // Comparison operators
  //----------------------------------------------------------------------

  bool operator==(const V& v) const 
    {
      return compare_all<AVAT::rel_eq<T> > (v);
    }

  bool operator==(T k) const 
    { 
      return compare_all<AVAT::rel_eq<T> > (VecDat<T,N>(k));
    }


  //----------------------------------------------------------------------
  // Comparison functions ... of geometric significance 
  //----------------------------------------------------------------------


  bool  all_l  (const V& v) const
    {
      return compare_all<AVAT::rel_l<T> >(v);
    }

  bool  all_le (const V& v) const
    {
      return compare_all<AVAT::rel_le<T> >(v);
    }

  bool  all_g  (const V& v) const
    {
      return compare_all<AVAT::rel_g<T> >(v);
    }

  bool  all_ge (const V& v) const
    {
      return compare_all<AVAT::rel_ge<T> >(v);
    }


  //----------------------------------------------------------------------
  // Assignment operators
  //----------------------------------------------------------------------

  void operator *=(T k) 
	{
		apply_assign<AVAT::mul_assign<T> > (VecDat<T,N>(k)); 
	}

  void operator /=(T k) 
    { 
      apply_assign<AVAT::div_assign<T> > (VecDat<T,N>(k)); 
		}

  void operator +=(T k) 
    { 
      apply_assign<AVAT::sum_assign<T> > (VecDat<T,N>(k)); 
		}

  void operator -=(T k) 
    { 
      apply_assign<AVAT::sub_assign<T> > (VecDat<T,N>(k)); 
		}

  void operator *=(const V& v) 
    { 
      apply_assign<AVAT::mul_assign<T> > (v); 
		}

  void operator /=(const V& v) 
    { 
      apply_assign<AVAT::div_assign<T> > (v); 
		}

  void operator +=(const V& v) 
    { 
      apply_assign<AVAT::sum_assign<T> > (v); 
		}

  void operator -=(const V& v) 
    { 
      apply_assign<AVAT::sub_assign<T> > (v); 
		}



  //----------------------------------------------------------------------
  // Unary operators on vectors
  //----------------------------------------------------------------------

	const V operator - () const
    {
      V v_new;
      v_new.apply<AVAT::mul<T> >(VecDat<T,N>(-1),*this);
      return v_new;
    }

  //----------------------------------------------------------------------
  // Binary operators on vectors
  //----------------------------------------------------------------------

  const V operator * (const V& v1) const
    {
      V v_new;
      v_new.apply<AVAT::mul<T> >(*this,v1);
      return v_new;
    }

  const V operator + (const V& v1) const
    {
      V v_new;
      v_new.apply<AVAT::add<T> >(*this,v1);
      return v_new;
    }

  const V operator - (const V& v1) const
    {
      V v_new;
      v_new.apply<AVAT::sub<T> >(*this,v1);
      return v_new;
    }

  const V operator / (const V& v1) const
    {
      V v_new;
      v_new.apply<AVAT::div<T> >(*this,v1);
      return v_new;
    }

  //----------------------------------------------------------------------
  // Binary operators on vector and scalar
  //----------------------------------------------------------------------

  const V operator * (T k) const
    {
      V v_new;
      v_new.apply<AVAT::mul<T> >(*this,VecDat<T,N>(k));
      return v_new;
    }
  

  const V operator / (T k) const
    {
      V v_new;
      v_new.apply<AVAT::div<T> >(*this,VecDat<T,N>(k));
      return v_new;      
    }

};


#endif

