#ifndef __ARITHVEC_H

// __ARITHVEC_H is defined in one of the two files included from this one
// so there is no need to also define it here!

#if defined IRIX
#include "ArithVec-using-scalars.h"
#else
#include "ArithVec-using-array.h"
#endif

//----------------------------------------------------------------------
// Input / output
//
// We don't even pretend to care about i/o performance.
// Therefore, we make just one function for output of all
// Vector types. This could be improved.
//----------------------------------------------------------------------
#include <iostream>

template <class T, class V, int N> 
inline std::ostream& operator<<(std::ostream&os, const ArithVec<T,V,N>& v)
{
  os << "[ ";
  for(int i=0;i<N;i++) os << v[i] << " ";
  os << "]";
  return os;
}

template <class T,class V, int N>
inline std::istream& operator>>(std::istream&is, ArithVec<T,V,N>& v)
{
  for(int i=0;i<N;i++) is>>v[i];
  return is;
}

template <class T,class V, int N>
inline T dot(const ArithVec<T,V,N>& v0, const ArithVec<T,V,N>& v1)
{
	T x = 0;
	for(int i=0;i<N;i++) x += v0[i]*v1[i];
	return x;
}

//----------------------------------------------------------------------
// operators on vector and scalar
// NOTE:
// It seems to be optimal to put the binary operators inside the
// ArithVec class template, but the operator functions whose 
// left operand is _not_ a vector cannot be inside, hence they
// are here.
// We need three operators for scalar * vector although they are
// identical, because, if we use a separate template argument for
// the left operand, it will match any type. If we use just T as 
// type for the left operand hoping that other built-in types will
// be automatically converted, we will be disappointed. It seems that 
// a float * ArithVec<float,Vec3f,3> function is not found if the
// left operand is really a double.
//----------------------------------------------------------------------

template<class T, class V, int N>
inline const V operator * (double k, const ArithVec<T,V,N>& v) 
{
	return v * k;
}

template<class T, class V, int N>
inline const V operator * (float k, const ArithVec<T,V,N>& v) 
{
	return v * k;
}

template<class T, class V, int N>
inline const V operator * (int k, const ArithVec<T,V,N>& v) 
{
	return v * k;
}

#endif

