#ifndef __MAT4X4_H
#define __MAT4X4_H

//#include "MathDefault.h"
#include "Common/ExceptionStandard.h"
#include "CGLA.h"
#include "Vec3f.h"
#include "Vec4f.h"
#include "ArithMat.h"

DERIVEEXCEPTION(Mat4x4fException, MotherException);
DERIVEEXCEPTION(Mat4x4fNotAffine, Mat4x4fException);
DERIVEEXCEPTION(Mat4x4fSingular,  Mat4x4fException);

class Mat4x4f: public ArithMat<Vec4f, Vec4f, Mat4x4f, 4>
{
public:
  
  Mat4x4f(Vec4f _a, Vec4f _b, Vec4f _c, Vec4f _d): 
    ArithMat<Vec4f, Vec4f, Mat4x4f, 4> (_a,_b,_c,_d) {}
  
  /// Construct the 0 matrix
  Mat4x4f() {}

	Mat4x4f(const float* sa): ArithMat<Vec4f, Vec4f, Mat4x4f, 4> (sa) {}

};


// MATRIX FUNCTIONS

Mat4x4f invert(const Mat4x4f&);

/// Create a rotation _matrix. Rotates about one of the major axes.
Mat4x4f rotation_matrix(CGLA::Axis axis, float angle);

/// Create a translation matrix
Mat4x4f translation_matrix(const Vec3f&);

/// Create a scaling matrix.
Mat4x4f scaling_matrix(const Vec3f&);

/// Create an identity matrix.
Mat4x4f identity_matrix();

/** Create a perspective matrix. Assumes the eye is at the origin and
    that we are looking down the negative z axis.
    
    ACTUALLY THE EYE IS NOT AT THE ORIGIN BUT BEHIND IT. CHECK UP ON
    THIS ONE */
Mat4x4f perspective_matrix(float d);


#endif







