#ifndef __UNITVECTOR_H
#define __UNITVECTOR_H

#include <math.h>
#include <iostream>
#include "CGLA.h"
#include "Vec3f.h"
#include "TableTrigonometry.h"

namespace TT=TableTrigonometry;

/** The UnitVector stores a unit length vector as two angles.

		A vector stored as two (fix point) angles is much smaller than
		vector stored in the usual way. On a 32 bit architecture this
		class should take up four bytes. not too bad. */
class UnitVector
{
	TT::Angle theta, phi;

	void encode(const Vec3f& v)
	{
		theta = TT::t_atan(sqrt(sqr(v[0])+sqr(v[1])), v[2]);
		phi   = TT::t_atan(v[0],v[1]);
	}
	
public:

	explicit UnitVector(const Vec3f& v) {encode(v);}
	explicit UnitVector(): theta(0), phi(0) {}

	float t() const {return TT::angle2float(theta);}
	float f() const {return TT::angle2float(phi);}

	operator Vec3f() const
	{
		float costf = TT::t_cos(theta);
		return Vec3f(TT::t_cos(phi)*costf , 
								 TT::t_sin(phi)*costf, 
								 TT::t_sin(theta));
	}
	bool operator==(const UnitVector& u) const
	{
		return theta == u.theta && phi == u.phi;
	}
};

inline std::ostream& operator<<(std::ostream& os, const UnitVector& u)
{
	os << "<v=" << u.t() << " h=" << u.f() << ">";
	return os;
}

#endif
