#ifndef __VEC2F_H
#define __VEC2F_H

#include <math.h>
#include "ArithVec.h"
#include "Vec2i.h"

class Vec2f: public ArithVec<float,Vec2f,2>
{
public:

  Vec2f() {}

  Vec2f(float _a,float _b): ArithVec<float,Vec2f,2>(_a,_b) {}

  explicit Vec2f(const Vec2i& v): ArithVec<float,Vec2f,2>(v[0],v[1]) {}
  
  float length() const 
    {
      return sqrt((*this)[0]*(*this)[0]+
                  (*this)[1]*(*this)[1]);
		}

	void round(int n) 
	{
		float d = (1.0/n);
		(*this)[0] = floor((*this)[0] * n + 0.5) * d;
		(*this)[1] = floor((*this)[1] * n + 0.5) * d;
	}

  void normalize() {*this/=length();}

};

inline Vec2f orthogonal(const Vec2f& v) 
{
	return Vec2f(-v[1],v[0]);
}
inline float cross(const Vec2f& a, const Vec2f& b)
{
	return a[0]*b[1]-a[1]*b[0];
}

bool linear_combine(const Vec2f&,const Vec2f&,const Vec2f&,float&,float&);



#endif
