#ifndef __VEC3HF_H
#define __VEC3HF_H


/** The H in Vec3Hf stands for homogenous. 

*/

#include "Vec3f.h"
#include "Vec4f.h"

class Vec3Hf: public Vec4f
{
public:
  
  /// Construct a (0,0,0,1) homogenous Vector
  Vec3Hf(): Vec4f(0,0,0,1.0) {}

  /// Construct a homogenous vector (a,b,c,1)
  Vec3Hf(float _a, float _b, float _c): Vec4f(_a,_b,_c,1.0) {}

  /// Construct a homogenous vector
  Vec3Hf(float _a, float _b, float _c, float _d): Vec4f(_a,_b,_c,_d) {}

  /// Construct a homogenous vector from a non-homogenous.
  explicit Vec3Hf(const Vec3f& v): Vec4f(v[0],v[1],v[2],1.0) {}

  /// Construct a homogenous vector from a non-homogenous.
  explicit Vec3Hf(const Vec3f& v,float _d): Vec4f(v[0],v[1],v[2],_d) {}

  /**  Construct a homogenous vector from a Vec4f.
       Notice that this constructor does not have an explicit 
       prepended. That is because matrix operations will
       produce vec4's and we usually want them assigned to 
       homogenous vectors and explicitly casting would
       be a bother. */
  Vec3Hf(const Vec4f& v): Vec4f(v) {}

  /// Divide all coordinates by the fourth coordinate
  void de_homogenize();
  
};


inline void Vec3Hf::de_homogenize() 
{
  (*this)[0] = (*this)[0]/(*this)[3];
  (*this)[1] = (*this)[1]/(*this)[3];
  (*this)[2] = (*this)[2]/(*this)[3];
  (*this)[3] = 1.0;
}

#endif

