#ifndef __Vec3d_H
#define __Vec3d_H

#include <math.h>
#include "ArithVec.h"
#include "Vec3i.h"
#include "Vec3f.h"

class Vec3d: public ArithVec<double,Vec3d,3>
{
  friend double dot(const Vec3d&,const Vec3d&);  
  friend Vec3d cross(const Vec3d&,const Vec3d&);

public:

  Vec3d(){}
  Vec3d(double a, double b, double c): ArithVec<double,Vec3d,3>(a,b,c) {}

  explicit Vec3d(double a): ArithVec<double,Vec3d,3>(a,a,a) {}
  explicit Vec3d(const Vec3i& v): ArithVec<double,Vec3d,3>(v[0],v[1],v[2]) {}
  explicit Vec3d(const Vec3f& v): ArithVec<double,Vec3d,3>(v[0],v[1],v[2]) {}
  
  double length() const 
    {
      return sqrt((*this)[0]*(*this)[0]+
                  (*this)[1]*(*this)[1]+
                  (*this)[2]*(*this)[2]);
    }
    
  void normalize() 
    {
      (*this) /= length();
    }
	  
  /** Get the vector in spherical coordinates.
      The first argument (theta) is inclination from the vertical axis.
      The second argument (phi) is the angle of rotation about the vertical 
      axis. The third argument (r) is the length of the vector. */
  void get_spherical( double&, double&, double& ) const;

  /** Assign the vector in spherical coordinates.
      The first argument (theta) is inclination from the vertical axis.
      The second argument (phi) is the angle of rotation about the vertical 
      axis. The third argument (r) is the length of the vector. */
  bool set_spherical( double, double, double);

};


inline double dot( const Vec3d& x, const Vec3d& y ) 
{
  return x[0]*y[0] + x[1]*y[1] + x[2]*y[2];
}


inline Vec3d cross( const Vec3d& x, const Vec3d& y ) 
{
  return Vec3d( x[1] * y[2] - x[2] * y[1], 
                x[2] * y[0] - x[0] * y[2], 
                x[0] * y[1] - x[1] * y[0] );
}


#endif
