#ifndef __VEC3F_H
#define __VEC3F_H

#include <math.h>
#include "ArithVec.h"
#include "Vec3i.h"
#include "Vec3usi.h"

class Vec3Hf;

class Vec3f: public ArithVec<float,Vec3f,3>
{
	friend Vec3f cross(const Vec3f&,const Vec3f&);

public:

  Vec3f(){}
  Vec3f(float a, float b, float c): ArithVec<float,Vec3f,3>(a,b,c) {}

  explicit Vec3f(float a): ArithVec<float,Vec3f,3>(a,a,a) {}
  explicit Vec3f(const Vec3i& v): ArithVec<float,Vec3f,3>(v[0],v[1],v[2]) {}
  explicit Vec3f(const Vec3usi& v): ArithVec<float,Vec3f,3>(v[0],v[1],v[2]) {}
  explicit Vec3f(const Vec3Hf& v);
  
  float length() const 
    {
      return sqrt((*this)[0]*(*this)[0]+
                  (*this)[1]*(*this)[1]+
                  (*this)[2]*(*this)[2]);
    }
    
  void normalize() 
    {
      (*this) /= length();
    }
	  
  /** Get the vector in spherical coordinates.
      The first argument (theta) is inclination from the vertical axis.
      The second argument (phi) is the angle of rotation about the vertical 
      axis. The third argument (r) is the length of the vector. */
  void get_spherical( float&, float&, float& ) const;

  /** Assign the vector in spherical coordinates.
      The first argument (theta) is inclination from the vertical axis.
      The second argument (phi) is the angle of rotation about the vertical 
      axis. The third argument (r) is the length of the vector. */
  bool set_spherical( float, float, float);

};

inline Vec3f cross( const Vec3f& x, const Vec3f& y ) 
{
  return Vec3f( x[1] * y[2] - x[2] * y[1], 
                x[2] * y[0] - x[0] * y[2], 
                x[0] * y[1] - x[1] * y[0] );
}


#endif


