#include <assert.h>
#include "ActionHistory.h"

const int ActionHistory::MAX_STACK_DEPTH = 25;

void ActionHistory::un_add()
{
	assert(at_end());
	delete action_history.back();
	action_history.pop_back();
	action_iter = --action_history.end();
	--stack_depth;
}


void ActionHistory::add(Action* const act_ptr) 
{
	if(at_beginning)
		action_history.clear();
	else
		action_history.erase(++action_iter, action_history.end());
	
	action_iter = action_history.insert(action_history.end(), act_ptr);
	if(stack_depth<MAX_STACK_DEPTH) 
		++stack_depth;
	else
		action_history.pop_front();
	at_beginning = false;
}

	/** Move one action back. Don't undo. This is useful, if we are restoring
			the system state - and don't need to reverse the action - but want to
			be able to redo() the action later */
bool ActionHistory::go_back()
{
	if(at_beginning) return false;
	
	if(action_iter == action_history.begin())
		at_beginning = true;
	else
		action_iter--;
	stack_depth--;
	
	return true;
}

/// Move one step ahead in the queue and redo action.
bool ActionHistory::redo() 
{
	if(action_iter == action_history.end()) return false;

	if(!at_beginning)
		action_iter++;
	else
		at_beginning = false;
	stack_depth++;

	return (*action_iter)->apply();
}

bool ActionHistory::apply_last()
{
	assert(!at_beginning);
	Action* a = (*action_iter);
	return a->apply();
}
