#ifndef __ACTIONHISTORY_H
#define __ACTIONHISTORY_H

#include <list>
#include <iostream>

#include "Action.h"

class ActionHistory
{
public:
	static const int MAX_STACK_DEPTH;

private:
	int stack_depth;
	bool at_beginning;
	
	std::list<Action*> action_history;
	typedef std::list<Action*>::iterator ActionIter;
	std::list<Action*>::iterator action_iter;

public:

	void clear(ActionIter iter)
	{
		for(ActionIter i=iter;i != action_history.end(); ++i) 
			delete *i;
		action_history.erase(iter, action_history.end());
	}

	void clear() {clear(action_history.begin());}

	ActionHistory(): at_beginning(true), stack_depth(0)
	{
		action_iter = action_history.end();
	}

	~ActionHistory()
	{
		clear();
	}
	
	/** Add an action to the action queue. Truncates queue if action_iter
			is not at an end. */
	void add(Action* const act_ptr);

	/// Move one action back. 
	bool go_back();

	void un_add();

	/// Move one step ahead in the queue and redo action.
	bool redo();

	bool at_end() 
	{
		return ((action_iter == --action_history.end())
						&& !at_beginning);

	}

	bool apply_last();

	Action* get_last() {return (*action_iter);}

};

#endif
