#ifndef __ARRAYBUFFER_H
#define __ARRAYBUFFER_H

#include <assert.h>
#include <malloc.h>

/** This is a very simple template that wraps an array with
    some simple functionality. It purpose in life is to be  
    slightly more efficient than an STL vector for very simple
    temporary storage.

    You add at the end, and in the case of overflow it reallocates.
*/    
template<class T>
class ArrayBuffer
{
  T* data;
  int end_idx;
  int size;

public:

  /** Get a chunk of storage. (We don't use new, since we don't
      want initialization of the chunk. */
  ArrayBuffer(int i=8):end_idx(0),size(i) 
    {
      data = (T*) malloc(i*sizeof(T));
    }

  /// Free our chunk of memory
  ~ArrayBuffer() {free((void*)(data));}
    
  /// Non-const access to arbitrary element.
  T& get(int i) {return data[i];}

  /// const access to arbitrary element.
  const T & get(int i) const {return data[i];}

  /// Non-const access to arbitrary element.
  T& last() 
    {
      assert(end_idx < size && end_idx >=0);
      return data[end_idx];
    }
  
  /// const access to arbitrary element.
  const T& last() const 
    {
      assert(end_idx<size);
      return data[end_idx];
    }

  int no_last() const {return end_idx;}

  /** Add an element to the tail of the sequence. Expand array 
      if required. */
  void add_to_end() 
    {
      assert(end_idx<size);
      end_idx++;
      if(end_idx==size) 
        {
          data=(T*)realloc(data,(size=2*size)*sizeof(T));
          assert(data);
        }
    }
  
  /// Simply reset the index of the end element to 0.
  void clear() {end_idx=0;}

  /// Return the number of elements in the array buffer.
  int no_elements() const {return end_idx;}
};


#endif
