// This is a template implementation file. It is called CC but I
// may have to include it from a source file - hence the guards.
#ifndef __FASTSTACKED_MAP_CC
#define __FASTSTACKED_MAP_CC

#include "BinSave.h"
#include "FastStackedMap.h"

template<class T>
void StateElement<T>::copy_all(const DataList&    _data, 
															 const DataIterVec& _data_ptr, 
															 int _no_elements)
{
	no_elements = 0;
	data.clear();
	data_ptr.resize(_data_ptr.size());
	
	for(int i=0;i<_data_ptr.size(); i++)
		{
			if(DataConstIter(_data_ptr[i]) != _data.end())
				{
					data.push_back(*_data_ptr[i]);
					data_ptr[i] = data.end();
					--data_ptr[i];
					++no_elements;
				}
			else data_ptr[i] = data.end();
		}
	assert(no_elements == _no_elements);
	assert(no_elements == data.size());
	assert(_data.size() == data.size());
}


template<class T>
int FastStackedMap<T>::add(const T& x)
{
	data.push_back(x);
	data_ptr.push_back(data.end());
		--data_ptr.back();
		no_elements++;
		
		assert(no_elements == data.size());
		
		return max_idx()-1;
}

template<class T>
void FastStackedMap<T>::remove(int i) 
{
	idx_precondition(i);
	data.erase(data_ptr[i]);
	data_ptr[i] = data.end();
	no_elements--;
}

template<class T>
int FastStackedMap<T>::get_position(const T& x) const
{
	int N = max_idx();
	for(int i=0; i<N; i++)
		if(&(*data_ptr[i]) == &x) return i;
	return 0;
}
	
template<class T>
void FastStackedMap<T>::push() 
{
	data_stack.push_back(StateElement<T>(data, data_ptr, no_elements));
	if(stack_depth < MAX_STACK_DEPTH)
		stack_depth++;
	else
		data_stack.erase(data_stack.begin());
	assert(data_stack.size() <= MAX_STACK_DEPTH);
}

template<class T>
bool FastStackedMap<T>::pop() 
{
	if(stack_depth == 0) return false;
	
	copy_all(data_stack.back().data,
					 data_stack.back().data_ptr,
					 data_stack.back().no_elements);
	
	data_stack.pop_back();
	stack_depth--;
	return true;
}

template<class T>
int FastStackedMap<T>::get_first() const
{
	int id=0;
	while(id < max_idx() && 
				DataConstIter(data_ptr[id]) == data.end()) id++;
	return id;
}


template<class T>
bool FastStackedMap<T>::get_next(int& id) const
{
	idx_precondition(id);
	
	id++;
	while(DataConstIter(data_ptr[id]) == data.end()) 
		{
			id++;
			if(id == max_idx()) return false;
		}
	return true;
}


template<class T>
void FastStackedMap<T>::save(std::ostream& os) const
{
	int N = max_idx();
	BinSave::bsave_simple(os, N);
	BinSave::bsave_simple(os, no_elements);
	int j=0;
 	for(int i=0; i< N; i++)
		if(DataConstIter(data_ptr[i]) != data.end())
			{
				BinSave::bsave_simple(os, i);
				data_ptr[i]->save(os);
				++j;
			}
	assert(j==no_elements);
}
	
template<class T>
void FastStackedMap<T>::load(std::istream& is) 
{
	data.clear();
	data_ptr.clear();
	
	int N;
	BinSave::bload_simple(is, N);
	BinSave::bload_simple(is, no_elements);
	
	data_ptr.resize(N);
	
	int i;
	for(i=0;i<N; i++)
		data_ptr[i] = data.end();
	
	for(int j=0;j<no_elements; j++)
		{
			int i;
			BinSave::bload_simple(is,i);
			data.push_back(T(is));
			data_ptr[i] = data.end();
			--data_ptr[i];
		}
}

#endif
