#ifndef __FASTSTACKEDMAP_H
#define __FASTSTACKEDMAP_H

#include <assert.h>

#include <functional>
#include <vector>
#include <list>


template<class T>
struct StateElement
{
	typedef std::list<T> DataList;
	typedef typename DataList::iterator DataIter;
	typedef typename DataList::const_iterator DataConstIter;
	typedef std::vector<DataIter> DataIterVec;
	
	DataList   data;
	DataIterVec data_ptr;
	int no_elements;

	StateElement(const StateElement& se): data(), data_ptr()
	{
		copy_all(se.data, se.data_ptr, se.no_elements);
	}
	StateElement(): data(), data_ptr(), no_elements() {}
	StateElement(const DataList& _data, const DataIterVec& _data_ptr, 
							 int _no_elements): data(), data_ptr()
	{
		copy_all(_data, _data_ptr, _no_elements);
	}


	void copy_all(const DataList&    _data, 
								const DataIterVec& _data_ptr, 
								int _no_elements);
};

	
	
template<class T>
class FastStackedMap: private StateElement<T>
{
	static int MAX_STACK_DEPTH;

	typedef std::list<StateElement<T> > StateStack;
	StateStack data_stack;
	int stack_depth;

public:

	FastStackedMap(): stack_depth(0) {}
	// FastStackedMap(std::istream& is): stack_depth(0), StateElement<T>(is) {}

	
	int max_idx() const {	return data_ptr.size(); }
	
	int size() const {return no_elements;}
	
#define idx_precondition(i) assert(i>=0);\
                            assert(i<max_idx());\
                            assert(DataConstIter(data_ptr[i])!=data.end());

	bool valid_idx(int id) const
	{
		if(id>=0 && id<max_idx() && 
			 DataConstIter(data_ptr[id]) != data.end()) return true;
		return false;
	}


public:


	int add(const T& x);
	void remove(int i);

	void remove(const T& x) { remove(get_position(x)); }

	int get_position(const T& x) const;

	const T& operator[](int id) const 
	{
		idx_precondition(id);
		return *data_ptr[id];
	}

	T& operator[](int id) 
	{
		idx_precondition(id);
		return *data_ptr[id];
	}

	void push();
	
	bool pop();

	int get_first() const;

	bool get_next(int& id) const;

	template<class FUNC>
	bool find_element(FUNC f, int& i) const
	{
		for(i= get_first(); i<max_idx(); get_next(i)) 
			if (f(*data_ptr[i])) return true;
		return false;
	}

	template<class FUNC>
	void for_all_elements(FUNC f)
	{
		for(int i= get_first(); i<max_idx(); get_next(i)) 
			f(*data_ptr[i]);
	}

	void save(std::ostream& os) const;

	void load(std::istream& is);
};
 
#endif




