#ifndef __HASHKEY_H
#define __HASHKEY_H

#include <stdlib.h>
#include <limits.h>
#include "CGLA/Vec3uc.h"
#include "CGLA/Vec3usi.h"

extern int randoms1[UCHAR_MAX];
extern int randoms2[UCHAR_MAX];
extern int randoms3[UCHAR_MAX];


struct HashKey3uc
{
	Vec3uc key;

	HashKey3uc() {}
	HashKey3uc(Vec3uc _key): key(_key) {}
	HashKey3uc(Vec3i _key): key(_key) {}

	int hash(int use_size) const
	{
		return int((randoms1[key[0]] >> (key[1]&0x0f)) +
							 (randoms2[key[1]] >> (key[2]&0x0f)) +
							 (randoms3[key[2]] >> (key[0]&0x0f))) & (use_size-1);
	}
	
	bool operator==(const HashKey3uc& k2) const {return key==k2.key;}
	bool operator!=(const HashKey3uc& k2) const {return !(key==k2.key);}
};

struct HashKey3usi
{
	Vec3usi key;

	HashKey3usi() {}
	HashKey3usi(Vec3usi _key): key(_key) {}
	HashKey3usi(Vec3i _key): key(_key) {}

	int hash(int use_size) const
	{
		return int(
   			((randoms1[key[0]&0xff00>>8] * randoms2[key[1]&0xff] >> (key[2]&0x0f))
			 + (randoms2[key[1]&0xff00>>8] * randoms1[key[2]&0xff] >> (key[0]&0x0f))
			 + (randoms3[key[2]&0xff00>>8] * randoms3[key[0]&0xff] >> (key[1]&0x0f)))
  	 	 & (use_size-1));
	}
	
	bool operator==(const HashKey3usi& k2) const {return key==k2.key;}
	bool operator!=(const HashKey3usi& k2) const {return !(key==k2.key);}
};

struct HashKey1c
{
	unsigned char key;

	HashKey1c() {}
	HashKey1c(unsigned char _key): key(_key) {}

	int hash(int use_size) const
	{
		return int(randoms1[key] & (use_size-1));
	}
	
	bool operator==(const HashKey1c& k2) const {return key==k2.key;}
	bool operator!=(const HashKey1c& k2) const {return !(key==k2.key);}
};

#endif
