#ifndef __DEPTHMAP_H
#define __DEPTHMAP_H

#include <GL/gl.h>
#include <assert.h>
#include "Common/CommonDefs.h"

/** A Depthmap is used to store values from the Z-buffer.
 */
class DepthMap
{
  /// bitmap containing z values
  float *bitmap;
  
  /// Width and height of map
  int width, height;
  
  /// distance from eye to near and far clipping planes
  float near, far;

public:
  
  /// Create a DepthMap of specified size
  DepthMap(int _width, int _height,
           float _near, float _far): 
    width(_width), height(_height), near(_near), far(_far)
    {
      bitmap = new float[_width*_height];
      memset(bitmap, 0, sizeof(bitmap));
    }
  
  /// Destroy a DepthMap
  ~DepthMap() 
    {
      delete [] bitmap;
    }    
  
  /// Return a pointer to the first byte of the DepthMap
  float* get_bitmap() { return bitmap;}

  float get_far() const {return far;}
  float get_near() const {return near;}

  float depthpixel(int x, int y) const
    {
      assert(y>=0 && y<height && x >=0 && x<width);
      float z_screen = bitmap[width*y + x];
      return -(far*near)/(z_screen*(far-near) - far);
    }

  void acquire()
    {
      glReadPixels(0, 0, width, height, GL_DEPTH_COMPONENT, GL_FLOAT, bitmap);
    }
  
  int write_map(const char* filename) const;

    
};

#endif
