#ifndef __EYE_H
#define __EYE_H

#include "CGLA/Vec3f.h"
#include "CGLA/Vec3Hf.h"
#include "Math/Transformation.h"
#include "Math/MatrixTransformation.h"
#include "Math/AffineMatrixTransformation.h"
#include "Math/PerspectiveTransformation.h"

/** This class represents an eye and the image plane.
    There are methods for changing viewing parameters, computing the 
    directrion of a viewing ray and performing perspective projection
    (includes viewing transformation and projection). 
    It seems that this class must be useful, because it has survived
    several projects almost unchanged. (I probably should get rid
    of the public variables though */
class Eye
{
  ///
  AffineMatrixTransformation vtm;
  ///
  PerspectiveTransformation ptm;
  ///
  MatrixTransformation mtm;

public:
  ///
  Vec3f eye_position;
  ///
  Vec3f focus_point;  
  ///
  Vec3f up;  
  ///
  Vec3f line_of_sight;

  ///
  Vec3f nv,uv,vv;

  ///
  float focal_dist;    
  
  ///
  void recalculate();

public:
  ///
  Eye(const Vec3f& ep, const Vec3f& fp, const Vec3f& u, const float fd):
    eye_position(ep), focus_point(fp), up(u), focal_dist(fd)
    {
      recalculate();
    };
  ///
  Eye(const Vec3f& ve, const Vec3f& vf, 
      const int d,const Vec3f& u);

  ///
  void set_eye_position(const Vec3f& v);
  ///
  void set_focus_point(const Vec3f& v);
  ///
  void set_focal_dist(const float d);
  ///
  void set_up(const Vec3f& v);  
  ///
  float zoom(const float fac);
  ///
  float rotate_horiz(float a);
  ///
  int rotate_vert(float a);

  ///
  float set_horiz_angle(float a);
  ///
  int set_vert_angle(float a);

  ///
  float get_horiz_angle()
    {
      float t,f,l;
      Vec3f r = - line_of_sight;
      r.get_spherical(t,f,l);
      return f;
    };
  
  ///
  float get_vert_angle()
    {
      float t,f,l;
      Vec3f r = - line_of_sight;
      r.get_spherical(t,f,l);
      return t;
    };

  ///
  void perspective(const Vec3Hf& p, float& x, float& y)
    {
      Vec3Hf p2 = mtm(p);
      p2.de_homogenize();
      x = p2[0];
      y = p2[1];
    };

  ///
  Vec3f get_ray_direction(const float ipx, const float ipy) const
    {
      Vec3f dir(nv);
      dir *= focal_dist; 
      dir += uv*ipx;
      dir += vv*ipy;
      dir.normalize();
      return dir;
    };
};

#endif
