#ifndef __RENDERCANVAS_H
#define __RENDERCANVAS_H

#include "Common/CommonDefs.h"
#include "ColorVector.h"



/** An abstract class containing the interface for drawing
    pixels to some sort of canvas. Non-abstract
    classes should be derived from this one and override the pure virtual 
    function draw_pixel to facilitate pixel drawing in the way supported by 
    the kind graphics system that the derived class represents. */
class RenderCanvas
{
protected:
  
  /// Width and height of canvas
  int width, height;

  /// Half width and height of canvas.
  float xw2,yw2;
  
public:

  /// Returns width of canvas
  int get_width() const {return width;}

  /// Returns height of canvas
  int get_height() const {return height;}
  
  /// 2D transform from window coordinates to image coordinates
  virtual void window_coords2image_coords(float x, float y, 
                                          float& ipx, float& ipy) const = 0;
  
  /// Constructs a canvas with specified dimensions
  RenderCanvas(int w, int h): width(w), height(h)
    {
      xw2 = float(width) / 2.0;
      yw2 = float(height) / 2.0;
    }
    
  /// Draw a pixel to the canvas.
  virtual void draw_pixel(int x, int y, const ColorVector& cv) = 0;
};

#endif









