/*
 * 04251 Computer Graphics and CAD (Grafisk databehandling og CAD)
 *
 */

#include "SFRotation.h"
#include "CGLA/Vec3f.h"

SFRotation::SFRotation() {
	setValue(0.0f, 0.0f, 1.0f, 0.0f);
}

SFRotation::SFRotation(float x, float y, float z, float rot) {
	setValue(x, y, z, rot);
}

SFRotation::SFRotation(float value[4]) {
	setValue(value);
}

////////////////////////////////////////////////
//	set value
////////////////////////////////////////////////
void SFRotation::setValue(float x, float y, float z, float rot) {
	quat[0] = x;
	quat[1] = y;
	quat[2] = z;
	quat[3] = rot;
}

void SFRotation::setValue(float value[4]) {
	quat[0] = value[0];
	quat[1] = value[1];
	quat[2] = value[2];
	quat[3] = value[3];
}

////////////////////////////////////////////////
//	get value
////////////////////////////////////////////////
void SFRotation::getValue(float value[4]) const
{
	value[0] = quat[0];
	value[1] = quat[0];
	value[2] = quat[0];
	value[3] = quat[0];
}

// Build a rotation matrix, given a quaternion rotation.
void SFRotation::getMatrix(float *matrix) const
{
  matrix[0] = 1.0f - 2.0f * (quat[1] * quat[1] + quat[2] * quat[2]);
  matrix[1] = 2.0f * (quat[0] * quat[1] - quat[2] * quat[3]);
  matrix[2] = 2.0f * (quat[2] * quat[0] + quat[1] * quat[3]);
  matrix[3] = 0.0f;
  
  matrix[4] = 2.0f * (quat[0] * quat[1] + quat[2] * quat[3]);
  matrix[5] = 1.0f - 2.0f * (quat[2] * quat[2] + quat[0] * quat[0]);
  matrix[6] = 2.0f * (quat[1] * quat[2] - quat[0] * quat[3]);
  matrix[7] = 0.0f;
  
  matrix[8] = 2.0f * (quat[2] * quat[0] - quat[1] * quat[3]);
  matrix[9] = 2.0f * (quat[1] * quat[2] + quat[0] * quat[3]);
  matrix[10] = 1.0f - 2.0f * (quat[1] * quat[1] + quat[0] * quat[0]);
  matrix[11] = 0.0f;
  
  matrix[12] = 0.0f;
  matrix[13] = 0.0f;
  matrix[14] = 0.0f;
  matrix[15] = 1.0f;
}

// Multiply two quaternions.
void SFRotation::mult(SFRotation q1, SFRotation q2)
{
	Vec3f v1(q1[0], q1[1], q1[2]);
	Vec3f v2(q2[0], q2[1], q2[2]);

	v1 *= q2[3];
	v2 *= q1[3];

	Vec3f v3(q1[0], q1[1], q1[2]);
	Vec3f v4(q2[0], q2[1], q2[2]);
	Vec3f v5 = cross(v4, v3);

	Vec3f v6 = v1 + v2;
	v6 += v5;

	quat[0] = v6[0];
	quat[1] = v6[1];
	quat[2] = v6[2];
	quat[3] = q1[3] * q2[3] - dot(v3, v4);
}
