#ifndef __ABSTRACTPLANE_H
#define __ABSTRACTPLANE_H

#include "Common/CommonDefs.h"
#include "HesseNormalForm.h"
#include "CGLA/Vec3f.h"
#include "Ray.h"

/** This class is the ancestor of all plane classes.  
    The only pure virtual function is the `n_dot' function
    that does dot product which is implemented differently in the various 
    derived classes */
class AbstractPlane: public HesseNormalForm
{

protected:

  /// Position in space
  Vec3f p;
  /** the dot product of the normal and the position in space
      times -1 */
  float d;
  /// computes dot product
  virtual float n_dot(const Vec3f&) const = 0;

public:

  /// Distance from point to plane 
  float distance(const Vec3f&) const;

  /// Intersection with ray
  bool intersection(Ray&) const;

  /// Intersection with ray
  bool intersection(Ray&, float&) const;
  
  /// Intersection with box
  bool box_intersect(const Vec3f&, const Vec3f&) const;

  /// The curvature at a point.
  float curvature(const Vec3f& dummy) const {return 0;}

	float foot_point(const Vec3f& p0, Vec3f& _n) const
	{
		_n = normal(p0);
		return distance(p0);
	}
};

/// Distance from point to plane
inline float AbstractPlane::distance(const Vec3f& v) const 
{
  return (n_dot(v)+d);
}

#endif
