#ifndef MATRIXTRANSFORMATION_H
#define MATRIXTRANSFORMATION_H

#include "CGLA/Mat4x4f.h"
#include "Transformation.h"

class MatrixTransformation: public Transformation
{
  ///
  friend inline std::ostream& operator << (std::ostream& os, 
                                           const MatrixTransformation& mt);


  ///
  Mat4x4f mtm;
  
protected:
  
  ///
  MatrixTransformation(const MatrixTransformation& mt1,
		       const MatrixTransformation& mt2):		       
    Transformation(mt1,mt2)
      {mtm = mt2.mtm * mt1.mtm;};
  
  ///
  void init_mtm(const Mat4x4f& m) {mtm = m;}
  
public:
  
  ///
  MatrixTransformation()
    {
      mtm  = identity_matrix();
    }
  
  ///
  Vec3Hf operator() (const Vec3Hf& v) const { return mtm * v; }


//   Ray operator() (const Ray& r) const 
//     { 
//       Vec3HF v = r.get_increment();
//       v[3] = 0;            // Set homogenous coordinate to nil
//       cout << "v = " << v << endl;
//       Vec3HF v2 = mtm * v; // Transform direction (no translation)
//       v2[3] = 1;           // Ensure homogenous coordinate is one
//       cout << "v2 = " << v2 << endl;
//       Vec3HF p2 = mtm * r.get_position(); // Transform position.      
//       return Ray(p2,v2);
///     }

  
  MatrixTransformation operator & (const MatrixTransformation& t) const
    {
      return MatrixTransformation(*this,t);
    }
  
  ///
  void concat (const MatrixTransformation& mt)
    {
      Transformation::concat(mt);
      mtm = mt.mtm * mtm;
    }
  
  ///
  void operator &= (const MatrixTransformation& mt)
    {
      concat(mt);
    }
  
  ///
  void print_trans(std::ostream& os) const
    {
      Transformation::print_trans(os);
      os << "\nTransformation matrix\n\n" << mtm;
    }
};




/******************************************************************************
 *                                                                           *
 * inline ostream& operator<<(ostream& os, const MatrixTransformation& mt)   *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/

inline std::ostream& operator<<(std::ostream& os, 
                                const MatrixTransformation& mt)
{
  mt.print_trans(os); 
  return os;
}


#endif
