#include <math.h>

#include "Common/CommonDefs.h"
#include "CGLA/Vec3f.h"
#include "Plane.h"


Plane::Plane(const Vec3f& _p, const Vec3f& a, const Vec3f& b): 
	is_bounded(false)
{
  
  /*  The plane is defined as follows:
   *   
   *   a is a point on the plane
   * 
   *   The normal is the cross product of b-a and c-a
   *
   *     c
   *     |
   *     |
   *     |
   *     a---b
   *
   */
	p=_p; 
	n = cross( a-_p , b-_p);
  n.normalize();
  d = - n_dot(p);
}

Plane::Plane(const Vec3f& _p, const Vec3f& _n): 
	is_bounded(false)
{
	n = _n;
  n.normalize();
	p = _p;
  d = -n_dot(p);
}

Plane::Plane(const Vec3f& _p, const Vec3f& _n, float _bbox_side_len): 
	is_bounded(true), bbox_side_len(_bbox_side_len/2.0) 
{
	n = _n;
  n.normalize();
	p = _p;
  d = -n_dot(p);
}


bool Plane::bounding_box(Vec3f& c0,Vec3f& c7) const 
{
	if(is_bounded)
		{
			Vec3f v(bbox_side_len,bbox_side_len,bbox_side_len);
			c0 = p - v;
			c7  =p + v;
			return true;
		}
	return false;
}









