#include <iostream>
#include "Ray.h"

using namespace std;

bool Ray::box_intersection(const Vec3f& corn0 ,const Vec3f& corn7)
{
	enum {RIGHT=0, LEFT=1, MIDDLE=2};

	// Find candidate planes; this loop can be avoided if
	// rays cast all from the eye(assume perpsective view) 
	bool inside = true;
	Vec3i quadrant;
	Vec3f candidatePlane;
	for (int i=0; i<3; i++)
		if(origin[i] < corn0[i]) 
			{
				quadrant[i] = LEFT;
				candidatePlane[i] = corn0[i];
				inside = false;
			}
		else if (origin[i] > corn7[i]) 
			{
				quadrant[i] = RIGHT;
				candidatePlane[i] = corn7[i];
				inside = false;
			}
		else	
			{
				quadrant[i] = MIDDLE;
			}

	// Ray origin inside bounding box 
	if(inside)
		{
			position = origin;
			t = 0.0;
			return true;
		}
	float maxT[3];
	// Calculate T distances to candidate planes 
	for (int i = 0; i < 3; i++)
		if (quadrant[i] != MIDDLE && direction[i] !=0.)
			maxT[i] = (candidatePlane[i]-origin[i]) / direction[i];
		else
			maxT[i] = -1.;
  
	// Get largest of the maxT's for final choice of intersection 
	int whichPlane = 0;
	for (int i = 1; i < 3; i++)
		if (maxT[whichPlane] < maxT[i])
			whichPlane = i;

	// Check final candidate actually inside box 
	if (maxT[whichPlane] < 0.) 
		return false;
  	
	Vec3f _position;
	for (int i = 0; i < 3; i++)
		if (whichPlane != i) 
			{
				_position[i] = origin[i] + maxT[whichPlane] *direction[i];
				if (_position[i] < corn0[i] || _position[i] > corn7[i])
					return false;
			}  
		else
			{
				_position[i] = candidatePlane[i];
			}
	position = _position;
	t = maxT[whichPlane];
	return true;				// ray hits box 
}
