#include <algorithm>
#include "Sphere.h"

using namespace std;


float Sphere::foot_point(const Vec3f& p, Vec3f& grad) const
{
	grad = p - centre;
	float grad_len = grad.length();
	float len = grad_len - radius;
	if(grad_len > 0) 
		grad /= grad_len;
	return len;
}





bool Sphere::intersection(Ray&r) const
{
  Vec3f sphere2ray_pos = r.get_origin() - centre;
  
  double A = 1;
  double B = 2 * dot(sphere2ray_pos, r.get_direction());
  double C = dot(sphere2ray_pos, sphere2ray_pos) - radius * radius;

  double Disc = B*B-4*A*C;

  if (Disc >= 0)
    {
      if (Disc == 0)
        r.set_parameter(- B / 2 * A);
      else
        {
          double Disc_sqrt = sqrt(Disc);
          double t1 = (-B+Disc_sqrt)/2.0*A;
          double t2 = (-B-Disc_sqrt)/2.0*A;
          
          double t;
          if((t=min(t1,t2))>0)
            r.set_parameter(t);
          else if((t=max(t1,t2))>0)
            r.set_parameter(t);
        }
      return true;
    }
  else
    return false;
}

// This code is by Arvo (Graphics Gems I)
bool Sphere::box_intersect(const Vec3f& Bmin,
                           const Vec3f& Bmax) const
{
  float dmax = 0;
  float dmin = 0;
  
  // For x, y and z ... i.e. we project the sphere onto
  // each of the three major axes, and we treat the problem 
  // separately for each axis.
  for( int i = 0; i < 3 ; i++ ) 
    {
      // Compute the distance from the sphere centre to the extremal
      // points of the box.
      float a = sqr( centre[i] - Bmin[i] );
      float b = sqr( centre[i] - Bmax[i] );

      dmax += max( a, b );

      if( centre[i] < Bmin[i] ) 
        dmin += a; 
      else if( centre[i] > Bmax[i] ) 
        dmin += b;
    }
  
  float  r2 = sqr( radius );
  if( dmin <= r2 && r2 <= dmax ) 
    return( true );
  
  return( false );
} 
