#ifndef __TRANSFORMATION_H
#define __TRANSFORMATION_H

#include <list>
#include <algobase.h>
#include <iostream>
#include "Common/CommonDefs.h"
#include "Common/History.h"
#include "CGLA/Vec3Hf.h"
#include "CGLA/Mat4x4f.h"

///
typedef enum TRANSFORMATIONS 
{
  ROTATE,TRANSLATE,SCALE,IDENTITY,
  BASISTRANS,PERSPECTIVE,FUNCTION
} ///
 c_trans;

///
const char trans_names[7][20] = {"ROTATE","TRANSLATE","SCALE",
                                 "IDENTITY","BASISTRANS",
                                 "PERSPECTIVE","FUNCTION"};

/****************************************************************************
 * class TransformationDesc                                                 *
 *                                                                          *
 * This class represents the notion of a transformation history             *
 * item ... i.e. an operation such as a scaling, rotation etc. and          *
 * it's parameters.                                                         *
 ****************************************************************************/



/******************************************************************************
 *                                                                           *
 * class TransformationDesc                                                  *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/

class TransformationDesc
{
  ///
  friend inline std::ostream& operator << (std::ostream& os, 
                                           const TransformationDesc& th);
  
  ///
  c_trans op;
  ///
  float x,y,z;
public:
  ///
  TransformationDesc(){};

  ///
  TransformationDesc(c_trans opa, float xa, float ya, float za):
    op(opa), x(xa),y(ya),z(za) {};
};



/******************************************************************************
 *                                                                           *
 * inline ostream& operator <<(ostream& os, const TransformationDesc& th)    *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/

inline std::ostream& operator <<(std::ostream& os, 
                                 const TransformationDesc& th)
{
  os << trans_names[th.op] << "(" << th.x << "," << th.y << "," 
     << th.z << ")";
  return os;
}




/******************************************************************************
 *                                                                           *
 * class Transformation: public History<TransformationDesc>                  *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/

class Transformation: public History<TransformationDesc>
{
  
protected:
  
  ///
  Transformation(const Transformation& t1,
		 const Transformation& t2):
    History<TransformationDesc>(t1,t2){};
  
  ///
  Transformation(const TransformationDesc& td):
    History<TransformationDesc>(td){};

  ///
  Transformation():
    History<TransformationDesc>(TransformationDesc(IDENTITY,0,0,0)){};
  
public:

  /// Transform
  virtual Vec3Hf operator () (const Vec3Hf& v) const = 0;
  
  ///
  void concat(const Transformation& t)
  {
    add_to_hist(t);
  }
  
  ///
  void print_trans(std::ostream& os) const
  {
//     os << "\nHistory\n\n";
//     History<TransformationDesc>::print_hist(os);
  }
};



#endif




